package mpi.eudico.client.annotator.commands;

import mpi.eudico.client.annotator.interannotator.AnnotatorCompareUtil2;
import mpi.eudico.client.annotator.interannotator.CompareCombi;

/**
 * A command that calculates some kind of agreement value based on the ratio of the 
 * amount of overlap of two annotation and the total extent of the two annotations when merged.
 */
public class CompareAnnotationRatioMultiCommand extends AbstractCompareCommand {

	/**
	 * Constructor.
	 * 
	 * @param theName
	 */
	public CompareAnnotationRatioMultiCommand(String theName) {
		super(theName);
	}

	/**
	 * Just calls the super implementation {@link AbstractCompareCommand#execute(Object, Object[])} 
	 */
	@Override
	public void execute(Object receiver, Object[] arguments) {
		super.execute(receiver, arguments);
	}

	/**
	 * Compares the segmentation of two tiers by calculating the quotient of the time overlap of
	 * two annotations and the total extent of the two (merged) annotations.
	 * This is similar to the Compare Annotators function that has been part of ELAN for some time,
	 * but here only the average for two tiers is calculated and stored (no overview containing all annotations).
	 * 
	 * @version October 2014
	 */
	@Override
	protected void calculateAgreement() {
		// in the super class checks have been performed and combinations of tiers have been
		// created. Start the calculations right away.
		if (compareSegments.size() == 0) {
			logErrorAndInterrupt("There are no tier pairs, nothing to calculate.");
			return;
		}
		double totalAverage = 0.0d;
		int combiCount = 0;
		// starting at an arbitrary 30%
		float perCombi = 60f / compareSegments.size();
		AnnotatorCompareUtil2 compareUtil = new AnnotatorCompareUtil2();
		progressUpdate((int) curProgress, "Starting calculations...");
		for (CompareCombi cc : compareSegments) {
			double average = compareUtil.getAverageRatio(cc);
			cc.setOverallAgreement(average);
			totalAverage += average;
			combiCount++;
			
			curProgress += perCombi;
			progressUpdate((int) curProgress, null);
		}
		
		// hier... do something with average values?
//		if (combiCount > 0) {
//			System.out.println(String.format("Completed calculations of %d pairs of tiers.", combiCount));
//			System.out.println("Average ag: " + totalAverage / combiCount);
//		}
		progressComplete(String.format("Completed calculations of %d pairs of tiers.", combiCount));
	}

	
}
