package mpi.eudico.client.annotator.interannotator;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Dictionary;
import java.util.Hashtable;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.gui.multistep.MultiStepPane;
import mpi.eudico.client.annotator.gui.multistep.StepPane;

@SuppressWarnings("serial")
public class CustomizationStep extends StepPane {
	private CompareConstants.METHOD curMethod = null;
	
	private JPanel customPanel;
	private JSlider overlapSlider;
	private SliderListener sliderListener;
	private JTextField overlapTF;
	private Insets globalInsets;
	private int defPercentage = 60;
	private JCheckBox verboseOutCB;
	
	public CustomizationStep(MultiStepPane multiPane) {
		super(multiPane);
		
		initComponents();
	}
	
    /*
     * Creates all basic ui elements.
     */
    protected void initComponents() {
    	setLayout(new GridBagLayout());
    	customPanel = new JPanel(new GridBagLayout());
    	
    	globalInsets = new Insets(5, 10, 5, 10);
    	GridBagConstraints gbc = new GridBagConstraints();
    	gbc.anchor = GridBagConstraints.NORTHWEST;
    	gbc.fill = GridBagConstraints.BOTH;
    	gbc.weightx = 1.0;
    	gbc.weighty = 1.0;
    	gbc.insets = globalInsets;
    	add(customPanel, gbc);
    	updateLocale();
    }
    
    public void updateLocale() {
    	customPanel.setBorder(new TitledBorder(ElanLocale.getString("CompareAnnotatorsDialog.CustomizeStep.TitleBorder")));
    }
    
    
    private void updatePanel(CompareConstants.METHOD method) {
    	customPanel.removeAll();
    	if (method == CompareConstants.METHOD.MOD_KAPPA) {
    		// add label + slider + basic or extensive output
    		JLabel percentageLabel = new JLabel(ElanLocale.getString(
    				"CompareAnnotatorsDialog.CustomizeStep.PercentageLabel"));
    		overlapTF = new JTextField(4);
    		overlapTF.setEditable(false);
    		overlapTF.setText(String.valueOf(defPercentage));
    		overlapSlider = new JSlider(51, 100, defPercentage);
    		Dictionary<Integer, JComponent> labels = new Hashtable<Integer, JComponent>(10);
    		labels.put(51, new JLabel("51"));
    		labels.put(60, new JLabel("60"));
    		labels.put(70, new JLabel("70"));
    		labels.put(80, new JLabel("80"));
    		labels.put(90, new JLabel("90"));
    		labels.put(100, new JLabel("100"));
    		overlapSlider.setLabelTable(labels);
    		//overlapSlider.setLabelTable(overlapSlider.createStandardLabels(20));
    		overlapSlider.setPaintLabels(true);
    		//overlapSlider.setMajorTickSpacing(10);
    		//overlapSlider.setPaintTicks(true);
    		sliderListener = new SliderListener();
    		overlapSlider.addChangeListener(sliderListener);    		
    		overlapSlider.createStandardLabels(20);
    		verboseOutCB = new JCheckBox(ElanLocale.getString(
    				"CompareAnnotatorsDialog.CustomizeStep.VerboseOutput"), false);
    		
    		
    		GridBagConstraints gbc = new GridBagConstraints();
    		gbc.anchor = GridBagConstraints.NORTHWEST;
    		gbc.fill = GridBagConstraints.HORIZONTAL;
    		gbc.weightx = 1.0;
    		gbc.insets = globalInsets;
    		gbc.gridwidth = 2;
    		customPanel.add(percentageLabel, gbc);
    		
    		gbc.gridy = 1;
    		gbc.gridwidth = 1;
    		gbc.fill = GridBagConstraints.NONE;
    		gbc.weightx = 0.0;
    		gbc.anchor = GridBagConstraints.NORTHWEST;
    		customPanel.add(overlapTF, gbc);
    		gbc.gridx = 1;
    		gbc.fill = GridBagConstraints.HORIZONTAL;
    		gbc.weightx = 1.0;
    		gbc.anchor = GridBagConstraints.NORTHWEST;
    		customPanel.add(overlapSlider, gbc);
    		
    		gbc.gridx = 0;
    		gbc.gridwidth = 2;
    		gbc.gridy = 2;
    		gbc.insets =  new Insets(5, 0, 5, 10);
    		customPanel.add(verboseOutCB, gbc);
    		// filler
    		gbc.gridy = 3;
    		gbc.fill = GridBagConstraints.BOTH;
    		gbc.weightx = 1.0;
    		gbc.weighty = 1.0;
    		customPanel.add(new JPanel(), gbc);
    		// read preferences
    		Object sliderPref = Preferences.get(CompareConstants.OVERLAP_PERCENTAGE, null);
    		if (sliderPref instanceof Integer) {
    			int percentage = (Integer) sliderPref;
    			if (percentage >= 51 && percentage <= 100) {
    				overlapSlider.setValue(percentage);
    			}
    		}
    		Object verbosePref = Preferences.get(CompareConstants.OUTPUT_PER_FILE, null);
    		if (verbosePref instanceof Boolean) {
    			verboseOutCB.setSelected((Boolean) verbosePref);
    		}
    	} else {
    		if (overlapSlider != null) {
    			overlapSlider.removeChangeListener(sliderListener);
    		}
    	}
    	//curMethod = method;
    	//updateLocale();
    }

	@Override
	public String getStepTitle() {
		return ElanLocale.getString("CompareAnnotatorsDialog.CustomizeStep.Title");
	}

	@Override
	public void enterStepForward() {
		CompareConstants.METHOD method = (CompareConstants.METHOD) 
				multiPane.getStepProperty(CompareConstants.METHOD_KEY);
		if (method == CompareConstants.METHOD.CLASSIC || method == CompareConstants.METHOD.STACCATO) {
			curMethod = method;
			leaveStepForward();
			multiPane.nextStep();
			return;
		}
		if (method != curMethod) {
			updatePanel(method);			
		}
		curMethod = method;
		multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, true);
		multiPane.setButtonEnabled(MultiStepPane.PREVIOUS_BUTTON, true);
	}

	@Override
	public boolean leaveStepForward() {
		if (curMethod == CompareConstants.METHOD.MOD_KAPPA) {
			// store percentage; store flag for verbose output
			if (overlapSlider != null) {//should be superfluous
				multiPane.putStepProperty(CompareConstants.OVERLAP_PERCENTAGE, Integer.valueOf(overlapSlider.getValue()));
				Preferences.set(CompareConstants.OVERLAP_PERCENTAGE, Integer.valueOf(overlapSlider.getValue()), null);
			}
			if (verboseOutCB != null) {
				multiPane.putStepProperty(CompareConstants.OUTPUT_PER_FILE, Boolean.valueOf(verboseOutCB.isSelected()));
				Preferences.set(CompareConstants.OUTPUT_PER_FILE, Boolean.valueOf(verboseOutCB.isSelected()), null);
			}
		}
		return true;
	}

	@Override
	public void enterStepBackward() {
		if (curMethod == CompareConstants.METHOD.CLASSIC || curMethod == CompareConstants.METHOD.STACCATO) {
			leaveStepBackward();
			multiPane.previousStep();
		} else {
			multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, true);
			multiPane.setButtonEnabled(MultiStepPane.PREVIOUS_BUTTON, true);
		}
	}
    
	/**
	 * A listener for the percentage slider.
	 */
    private class SliderListener implements ChangeListener {

		@Override
		public void stateChanged(ChangeEvent ce) {
			// check the source if it can be ambiguous. Now assume the one slider fired the event.
			if (overlapSlider != null && overlapTF != null) {
				overlapTF.setText(String.valueOf(overlapSlider.getValue()));
			}
		}
    	
    }
}
