package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.List;

import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import mpi.eudico.client.annotator.Preferences;
import nl.mpi.lexan.annotyzers.helpers.Information;


/**
 * A panel to set the configuration settings
 * required for the annotyzer
 * 
 * @author aarsom
 */
public class AnnotyzerSettingsPanel extends JPanel implements ActionListener {
	private InterlinearMediator mediator;
	private List<Information> annotyzers;
	private JComboBox tyzerComboBox;
	private JPanel settingsPanel;
	
	private HashMap<String, Component> componentMap;
	
	
	public AnnotyzerSettingsPanel(InterlinearMediator mediator){
		super();
		this.mediator = mediator;
		annotyzers = mediator.listAnnotyzers();
		componentMap = new HashMap<String, Component>();
		initComponents();
	}
	
	private void initComponents(){
		if(annotyzers == null || annotyzers.size() <= 0){
			return;
		}
		
		tyzerComboBox = new JComboBox();
		Information annotyzer;
		Component comp;
		for(int i =0; i < annotyzers.size(); i++){
			annotyzer = annotyzers.get(i);
			comp = mediator.getConfigurationComponent(annotyzer);
			if( comp != null){
				tyzerComboBox.addItem(annotyzers.get(i).name);
				componentMap.put(annotyzers.get(i).name, comp);
			}
		}
		
		if(componentMap.size() > 0){
			settingsPanel = new JPanel();
			settingsPanel.setLayout(new GridBagLayout());
			
			setLayout(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.anchor = GridBagConstraints.NORTHWEST;
			gbc.fill = GridBagConstraints.NONE;
			gbc.insets = new Insets(2, 4, 2, 4);
			gbc.gridwidth = 2;
			add(new JLabel("Configure Annotyzer Settings"), gbc);
			
			gbc.gridy = 1;
			gbc.gridwidth = 1;
			add(new JLabel("Select a annotyzer"), gbc);
			
			gbc.gridx = 1;
			add(tyzerComboBox, gbc);
			
			gbc.gridx = 0;
			gbc.gridy = 2;
			gbc.gridwidth = 2;
			gbc.weightx = 1.0;
			gbc.weighty =1.0;
			gbc.fill = GridBagConstraints.BOTH;
			add(settingsPanel, gbc);
			
			tyzerComboBox.setSelectedIndex(-1);
			tyzerComboBox.addActionListener(this);
			
			readPreferences();
			
		}
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if(e.getSource() == tyzerComboBox){
			settingsPanel.removeAll();
			Component comp = componentMap.get(tyzerComboBox.getSelectedItem().toString());
			
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.anchor = GridBagConstraints.NORTHWEST;
			gbc.fill = GridBagConstraints.BOTH;
			gbc.weightx = 1.0;
			gbc.weighty =1.0;
			gbc.insets = new Insets(2, 4, 2, 4);
			settingsPanel.add(comp, gbc);
			
//			gbc.gridy = 1;
//			settingsPanel.add(new JPanel(), gbc);
			
			revalidate();
		}
	}
	
	private void readPreferences() {
		Object val = Preferences.get("AnnotyzerConfigurationPanel.CurrentAnnotyzer", 
				 mediator.getTranscription());
		if(val instanceof String){
			String tyzer = (String)val;
			for(int i =0; i < tyzerComboBox.getItemCount(); i++){
				if(tyzer.equals(tyzerComboBox.getItemAt(i).toString())){
					tyzerComboBox.setSelectedItem(tyzer);
					break;
				}
			}
		}
	}
	
	public void storePreferences() {
		if(tyzerComboBox.getSelectedItem() != null){
			Preferences.set("AnnotyzerConfigurationPanel.CurrentAnnotyzer", 
					tyzerComboBox.getSelectedItem().toString(), mediator.getTranscription());
		}
	}

}
