package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JScrollBar;

import nl.mpi.lexan.annotyzers.helpers.Position;
import nl.mpi.lexan.annotyzers.helpers.SuggestionSet;
import mpi.eudico.client.annotator.Constants;
import mpi.eudico.client.annotator.Preferences;
import mpi.eudico.client.annotator.interlinear.edit.event.IGTViewerChangeListener;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTAnnotation;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTDefaultModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTTier;
import mpi.eudico.client.annotator.interlinear.edit.AnnotTypeConfig;
import mpi.eudico.client.annotator.layout.InterlinearizationManager;
import mpi.eudico.client.annotator.util.AnnotationDataComparator;
import mpi.eudico.client.annotator.viewer.AbstractViewer;
import mpi.eudico.client.mediacontrol.ControllerEvent;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clom.Transcription;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.AlignableAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.event.ACMEditEvent;
import mpi.eudico.server.corpora.event.ACMEditListener;

public class InterlinearEditor extends AbstractViewer 
implements ComponentListener, ActionListener, ACMEditListener, AdjustmentListener, IGTViewerChangeListener {
	private InterlinearizationManager manager;
	private TranscriptionImpl transcription = null;
	
    private Font font;
    private FontMetrics metrics;
    private BufferedImage bi;
    private Graphics2D big2d;
    private final int MARGIN = 5;
    private int bpHeight = 40;
    
    private JPanel buttonPanel;
    private JButton interButton;
    private JCheckBox recursiveCB;
    
    private AnnotConfig activeConfig = null;
    private IGTViewer igtViewer = null;
    private IGTViewerModelImpl viewerModel = null;
    private JScrollBar viewerScrollBar = null;
    private int scrollBarWidth;

	
	public InterlinearEditor(InterlinearizationManager manager) {
		super();
		this.manager = manager;
		this.transcription = (TranscriptionImpl) manager.getTranscription();
		initViewer();
	}
	
	public void configsChanged() {
		paintBuffer();
	}
	
	public void setActiveConfiguration(AnnotConfig ac) {
		activeConfig = ac;
	}
	
	public AnnotConfig getActiveConfiguration() {
		return activeConfig;
	}
	
	
	private void initViewer() {
		setOpaque(true);
        font = Constants.DEFAULTFONT;
        setFont(font);
        metrics = getFontMetrics(font);
		setBackground(Color.WHITE);
		// set layout
		// add components
		buttonPanel = new JPanel();
		interButton = new JButton("Interlinearize");
		interButton.addActionListener(this);
		buttonPanel.add(interButton);
		add(buttonPanel);
		recursiveCB = new JCheckBox("Recursive");
		buttonPanel.add(recursiveCB);
		recursiveCB.addActionListener(this);
		//
		viewerModel = new IGTViewerModelImpl();
		initModel();
		igtViewer = new IGTViewer(viewerModel, manager.getMediator());
		add(igtViewer);
		setPreferredFontAndColorSettings();
		igtViewer.addViewerChangeListener(this);
		viewerScrollBar = new JScrollBar(JScrollBar.VERTICAL);
		add(viewerScrollBar);
		scrollBarWidth = viewerScrollBar.getPreferredSize().width;
		addComponentListener(this);
		viewerScrollBar.addAdjustmentListener(this);
		paintBuffer();
	}
	
	/**
	 * Sets the preferred font and color setting for the tiers
	 *                         							
	 */
	private void setPreferredFontAndColorSettings(){			
		// preferred tier colors
		HashMap tierColorMap = new HashMap<String, Color>();
		Object colorsObj = Preferences.get("TierColors", transcription);
		if (colorsObj instanceof Map) {
			Map colors = (Map) colorsObj;
			Iterator keyIt = colors.keySet().iterator();
			String name;
			TierImpl t;
			Color col;
			while (keyIt.hasNext()) {
				name = (String) keyIt.next();
				col = (Color) colors.get(name);
				t = (TierImpl) transcription.getTierWithId(name);
				if (t != null) {					
					tierColorMap.put(name, col);
				}
			}
		}
		
		igtViewer.setTierColorMap(tierColorMap);
		
		//preferred fonts
		Object fo = Preferences.get("TierFonts", transcription);
		if (fo instanceof HashMap) {
			igtViewer.setTierFontMap((HashMap) fo);
		}
		
		//preferred font size
		Object size = Preferences.get("TimeLineViewer.FontSize", transcription);
		if (size instanceof Integer) {
			igtViewer.setTierFontSize((Integer) size);
		}
	}

	private void initModel() {
		// get all annotations from all (toplevel) tiers, for the moment
		List<AlignableAnnotation> allAnns = new ArrayList<AlignableAnnotation>();
		TierImpl t;
		for (int i = 0; i < transcription.getTiers().size(); i++) {
			t = (TierImpl) transcription.getTiers().get(i);
			if (!t.hasParentTier()) {
				allAnns.addAll(t.getAnnotations());
			}
		}
		AnnotationDataComparator comparator = new AnnotationDataComparator();
		Collections.sort(allAnns, comparator);
		
		AlignableAnnotation aa;
		IGTDefaultModel rowModel;
		for (int i = 0; i < allAnns.size(); i++) {
			aa = (AlignableAnnotation) allAnns.get(i);
			rowModel = new IGTDefaultModel(aa);
			viewerModel.addRow(rowModel);
		}
	}
	
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		Graphics2D g2d = (Graphics2D) g;
//		g2d.setColor(getBackground());
//		g2d.fillRect(0, 0, getWidth(), getHeight());
//		if (bi != null) {
//			g2d.drawImage(bi, 0, bpHeight, null);
//		}
	}

	public void controllerUpdate(ControllerEvent event) {
		// TODO Auto-generated method stub

	}
	
	private void paintBuffer() {
        if ((getWidth() <= 0) || (getHeight() <= 0)) {
            return;
        }
//        int imageWidth = getWidth();
//        int imageHeight = getHeight() - bpHeight;
//        if ((bi == null) || (bi.getWidth() < imageWidth) ||
//                (bi.getHeight() < imageHeight)) {
//            bi = new BufferedImage(imageWidth, imageHeight,
//                    BufferedImage.TYPE_INT_RGB);
//            big2d = bi.createGraphics();
//            big2d.setFont(font);
//        }
//        big2d.setColor(Color.WHITE);
//        big2d.fillRect(0, 0, imageWidth, bi.getHeight());
//        
//        big2d.setColor(Color.BLACK);

        
        repaint();
	}
	
	/**
	 * For now, at any edit event, repaint
	 * @param e
	 */
	public void ACMEdited(ACMEditEvent e) {
		// TODO check the event type, and how to respond to each relevant event
		switch (e.getOperation()) {
		case ACMEditEvent.ADD_ANNOTATION_HERE:
			
            if (e.getInvalidatedObject() instanceof TierImpl &&
                    e.getModification() instanceof AbstractAnnotation) {
            	AbstractAnnotation newAnn = (AbstractAnnotation) e.getModification();
                TierImpl invTier = (TierImpl) e.getInvalidatedObject();
                TierImpl rootTier = invTier.getRootTier();// TODO the annotation can be on a root tier?
                String rootTierName = rootTier.getName();

                IGTDefaultModel rowModel;
                IGTTier igtTier;
                IGTAnnotation igtAnn;
                
                for (int i = 0; i < viewerModel.getRowCount(); i++) {
                	rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                	igtTier = rowModel.getRootRow();
                	if (igtTier.getTierName().equals(rootTierName)) {
                		List<IGTAnnotation> annos = igtTier.getAnnotations();
                		if (annos != null && annos.size() > 0) {
                			igtAnn = annos.get(0);
                			
                			if (igtAnn.getAnnotation().getBeginTimeBoundary() <= newAnn.getBeginTimeBoundary() && 
                					igtAnn.getAnnotation().getEndTimeBoundary() >= newAnn.getEndTimeBoundary()) {
                				// if this succeeds this generates an event 
                				rowModel.annotationAdded(newAnn);
                				break;
                			}
                		}
                	}
                }
            }
            
			break;
		case ACMEditEvent.ADD_ANNOTATION_AFTER:
			
            if (e.getInvalidatedObject() instanceof TierImpl &&
                    e.getModification() instanceof AbstractAnnotation) {
            	AbstractAnnotation newAnn = (AbstractAnnotation) e.getModification();
                TierImpl invTier = (TierImpl) e.getInvalidatedObject();
                AbstractAnnotation annBefore = (AbstractAnnotation) invTier.getAnnotationBefore(newAnn);
                if (annBefore != null && annBefore.getParentAnnotation() == newAnn.getParentAnnotation()) {
	                TierImpl rootTier = invTier.getRootTier();
	                String rootTierName = rootTier.getName();
	
	                IGTDefaultModel rowModel;
	                IGTTier igtTier;
	                IGTAnnotation igtAnn;
	                
	                for (int i = 0; i < viewerModel.getRowCount(); i++) {
	                	rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
	                	igtTier = rowModel.getRootRow();
	                	if (igtTier.getTierName().equals(rootTierName)) {
	                		List<IGTAnnotation> annos = igtTier.getAnnotations();
	                		if (annos != null && annos.size() > 0) {
	                			igtAnn = annos.get(0);
	                			
	                			if (igtAnn.getAnnotation().getBeginTimeBoundary() <= newAnn.getBeginTimeBoundary() && 
	                					igtAnn.getAnnotation().getEndTimeBoundary() >= newAnn.getEndTimeBoundary()) {
	                				// if this succeeds this generates an event 
	                				rowModel.annotationAddedAfter(newAnn, annBefore);
	                				break;
	                			}
	                		}
	                	}
	                }
                }// else warn??
            }
            
			break;
		case ACMEditEvent.ADD_ANNOTATION_BEFORE:

            if (e.getInvalidatedObject() instanceof TierImpl &&
                    e.getModification() instanceof AbstractAnnotation) {
            	AbstractAnnotation newAnn = (AbstractAnnotation) e.getModification();
                TierImpl invTier = (TierImpl) e.getInvalidatedObject();
                AbstractAnnotation annAfter = (AbstractAnnotation) invTier.getAnnotationAfter(newAnn);
                if (annAfter != null && annAfter.getParentAnnotation() == newAnn.getParentAnnotation()) {
	                TierImpl rootTier = invTier.getRootTier();
	                String rootTierName = rootTier.getName();
	
	                IGTDefaultModel rowModel;
	                IGTTier igtTier;
	                IGTAnnotation igtAnn;
	                
	                for (int i = 0; i < viewerModel.getRowCount(); i++) {
	                	rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
	                	igtTier = rowModel.getRootRow();
	                	if (igtTier.getTierName().equals(rootTierName)) {
	                		List<IGTAnnotation> annos = igtTier.getAnnotations();
	                		if (annos != null && annos.size() > 0) {
	                			igtAnn = annos.get(0);
	                			
	                			if (igtAnn.getAnnotation().getBeginTimeBoundary() <= newAnn.getBeginTimeBoundary() && 
	                					igtAnn.getAnnotation().getEndTimeBoundary() >= newAnn.getEndTimeBoundary()) {
	                				// if this succeeds this generates an event 
	                				rowModel.annotationAddedBefore(newAnn, annAfter);
	                				break;
	                			}
	                		}
	                	}
	                }
                } // else warn??
            }
            
			break;
        case ACMEditEvent.CHANGE_ANNOTATION_VALUE:
        	// modification object is null
            if (e.getInvalidatedObject() instanceof AbstractAnnotation) {
            	AbstractAnnotation a = (AbstractAnnotation) e.getInvalidatedObject();
                long bt = a.getBeginTimeBoundary();
                long et = a.getEndTimeBoundary();
                
                TierImpl invTier = (TierImpl) a.getTier();
                TierImpl rootTier = invTier.getRootTier();
                String rootTierName = rootTier.getName();
            	
                IGTDefaultModel rowModel;
                IGTTier igtTier;
                IGTAnnotation igtAnn;

                for (int i = 0; i < viewerModel.getRowCount(); i++) {
                	rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                	igtTier = rowModel.getRootRow();
                	if (igtTier.getTierName().equals(rootTierName)) {
                		List<IGTAnnotation> annos = igtTier.getAnnotations();
                		if (annos != null && annos.size() > 0) {
                			igtAnn = annos.get(0);
                			
                			if (igtAnn.getAnnotation().getBeginTimeBoundary() <= bt && 
                					igtAnn.getAnnotation().getEndTimeBoundary() >= et) {
                				// if this succeeds this generates an event 
                				rowModel.annotationValueChanged(a);
                				break;
                			}
                		}
                	}
                }
                
            }
            
            break;
            
        case ACMEditEvent.REMOVE_ANNOTATION:
        	// currently the transcription is the only information available in the event
        	// the deleted annotation needs to be accessible as well
            if (e.getInvalidatedObject() instanceof Transcription) {
                //System.out.println("Invalidated object: " + e.getInvalidatedObject());
            	
            	if (e.getModification() instanceof AbstractAnnotation) {
            		AbstractAnnotation remAnnotation = (AbstractAnnotation) e.getModification();
                    long bt = remAnnotation.getBeginTimeBoundary();
                    long et = remAnnotation.getEndTimeBoundary();
                    
                    TierImpl invTier = (TierImpl) remAnnotation.getTier();
                    TierImpl rootTier = invTier.getRootTier();
                    String rootTierName = rootTier.getName();
                	
                    IGTDefaultModel rowModel;
                    IGTTier igtTier;
                    IGTAnnotation igtAnn;

                    for (int i = 0; i < viewerModel.getRowCount(); i++) {
                    	rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                    	igtTier = rowModel.getRootRow();
                    	if (igtTier.getTierName().equals(rootTierName)) {
                    		List<IGTAnnotation> annos = igtTier.getAnnotations();
                    		if (annos != null && annos.size() > 0) {
                    			igtAnn = annos.get(0);
                    			
                    			if (igtAnn.getAnnotation().getBeginTimeBoundary() <= bt && 
                    					igtAnn.getAnnotation().getEndTimeBoundary() >= et) {
                    				// if this succeeds this generates an event 
                    				rowModel.annotationRemoved(remAnnotation);
                    				break;
                    			}
                    		}
                    	}
                    }
            	}

                
            }

            break;
            
        case ACMEditEvent.CHANGE_ANNOTATIONS:
        	// any number of annotations might have been changed. Assume a change in the current block only for now.
        	
        	if (viewerModel.getEditingRow() > -1) {
        		IGTDefaultModel rowModel = (IGTDefaultModel) viewerModel.getRowData(viewerModel.getEditingRow());
        		// reload the whole model
        	} else {
        		//Temp
        		while ( viewerModel.getRowCount() > 0) {
            		viewerModel.removeRow(viewerModel.getRowCount() - 1);
                 }
            	initModel();
            	igtViewer.recalculateAllRows();
        	}        	
        	break;
        	
        case ACMEditEvent.REMOVE_TIER:
        	Object obj  = e.getModification();
			if(obj instanceof TierImpl){
				TierImpl invTier = (TierImpl) obj;
                TierImpl rootTier = invTier.getRootTier();
                String rootTierName = rootTier.getName();
				IGTDefaultModel rowModel;
                IGTTier igtTier;
                for (int i = 0; i < viewerModel.getRowCount(); i++) {
                	rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                 	igtTier = rowModel.getRootRow();
                 	if (igtTier.getTierName().equals(rootTierName)) {
                 		//igtTier.removeChildTier(invTier.getName());
                 		rowModel.tierRemoved(invTier.getName());
                 		if(invTier == rootTier){
                 			viewerModel.removeRowData(rowModel);
                 		}
                 	}
                 }
                igtViewer.recalculateAllRows();
			}
			break;	
			
        case ACMEditEvent.ADD_TIER:
        	obj  = e.getModification();
			if(obj instanceof TierImpl){
				TierImpl invTier = (TierImpl) obj;
                if(invTier.hasParentTier()){
                	TierImpl rootTier = invTier.getRootTier();
                    String rootTierName = rootTier.getName();
    				IGTDefaultModel rowModel;
                    IGTTier igtTier;
                    for (int i = 0; i < viewerModel.getRowCount(); i++) {
                    	rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                     	igtTier = rowModel.getRootRow();
                     	if (igtTier.getTierName().equals(rootTierName)) {
                     		//igtTier.removeChildTier(invTier.getName());
                     		rowModel.tierAdded(invTier);
                     	}
                     }
                    igtViewer.recalculateAllRows();
                } else {
                	List<AlignableAnnotation> allAnns = new ArrayList<AlignableAnnotation>();
                	allAnns.addAll(invTier.getAnnotations());
                	
                	AlignableAnnotation aa;
            		IGTDefaultModel rowModel;
            		IGTDefaultModel newRowModel;
            		
            		int nextIndex = 0;
            		while(allAnns.size() > 0){
            			aa = (AlignableAnnotation) allAnns.get(0);
            			newRowModel = new IGTDefaultModel(aa);
            			
            			if(nextIndex >= viewerModel.getRowCount()-1){
            				viewerModel.addRow(newRowModel);	
            				nextIndex++;
            			} else {
            				for (int i = nextIndex; i < viewerModel.getRowCount(); i++) {
                            	rowModel = (IGTDefaultModel) viewerModel.getRowData(i);
                            	long beginTime = rowModel.getBeginTime();
                             	
                            	// begin time
                                if (aa.getBeginTimeBoundary() < beginTime) {
                                	viewerModel.insertRow(newRowModel, i);
                                	nextIndex = i+1;
                                	break;
                                } else if (aa.getBeginTimeBoundary() > beginTime) {
                                   continue;
                                }
                                
                                // begin time equal, compare end time
                                if (aa.getEndTimeBoundary() < rowModel.getEndTime()) {
                                	viewerModel.insertRow(newRowModel, i);
                                	nextIndex = i+1;
                                	break;
                                } else if (aa.getEndTimeBoundary() > rowModel.getEndTime()) {
                                	continue;
                                }
                             }
            			}
            		}
            		if(allAnns.size() > 0){
                    	igtViewer.recalculateAllRows();
                    }
                }
			}
			break;
			
			default:
				// do nothing
		}
		paintBuffer();
	}
	
	public void handleSuggestions(List<SuggestionSet> sugSets) {
		
	}
	
	public void updateSelection() {
		// TODO Auto-generated method stub

	}

	public void updateActiveAnnotation() {
		// TODO Auto-generated method stub

	}

	public void updateLocale() {
		// TODO Auto-generated method stub

	}

	public void preferencesChanged() {
		setPreferredFontAndColorSettings();
		
		repaint();
	}
	
	/**
	 * Updates the location and sizes of the components.
	 */
	public void componentResized(ComponentEvent e) {
		buttonPanel.setBounds(0, 0, getWidth(), bpHeight);
		igtViewer.setBounds(0, bpHeight , getWidth() - scrollBarWidth, getHeight() - bpHeight);
		igtViewer.recalculateAllRows();
		viewerScrollBar.setBounds(getWidth() - scrollBarWidth, bpHeight, scrollBarWidth, 
				getHeight() - bpHeight);
		paintBuffer();	
	}

	public void componentMoved(ComponentEvent e) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * Repaints.
	 */
	public void componentShown(ComponentEvent e) {
		paintBuffer();		
	}

	public void componentHidden(ComponentEvent e) {
			
	}

	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == interButton) {
			AnnotConfig ac = activeConfig;
			if (ac == null) {
				if (((InterlinearMediator) manager.getMediator()).getConfigurations().size() > 0) {
					ac = ((InterlinearMediator) manager.getMediator()).getConfigurations().get(0);
				}
			}
			if (ac == null) {
				return;
			}
						
			List<AnnotConfig> activeConfList = new ArrayList<AnnotConfig>();
			if(ac.isTypeMode()){
				activeConfList.addAll(((AnnotTypeConfig)ac).getTierConfigurations());
			} else {
				activeConfList.add(ac);
			}
			
			for(AnnotConfig conf : activeConfList){
				TierImpl tier = (TierImpl) transcription.getTierWithId(conf.getSource());
				if (tier != null) {
					int numAnn = tier.getNumberOfAnnotations();
					for(int i = 0; i < numAnn; i++) {
						Annotation a = (Annotation) tier.getAnnotations().get(i);
						Position pos = new Position();
						pos.tierId = conf.getSource();// To do: check whether source or destination needs to be passed
						//pos.tierId = ac.getDestTier();
						pos.beginTime = a.getBeginTimeBoundary();
						pos.endTime = a.getEndTimeBoundary();
						
						manager.getMediator().annotyze(pos);
					} // else warn
				} // else warn
			}
			
			((InterlinearMediator)manager.getMediator()).interLinearizeActionCompleted();
		
		} else if(e.getSource() == recursiveCB){
			((InterlinearMediator) manager.getMediator()).setRecursive(recursiveCB.isSelected());
		}
	}

	/**
	 * Inform the viewer of a scroll event
	 */
	public void adjustmentValueChanged(AdjustmentEvent e) {
		if (e.getSource() == viewerScrollBar) {
			int value = viewerScrollBar.getValue();
			igtViewer.setVerticalScrollValue(-value);
		}		
	}

	/**
	 * Updates the scrollbar after a change in the viewer.
	 */
	public void viewChanged() {
		int amount = igtViewer.getTotalHeight();
		int value = viewerScrollBar.getValue();
		viewerScrollBar.removeAdjustmentListener(this);
		int visAmount = (getHeight() - bpHeight);
		viewerScrollBar.setVisibleAmount(visAmount);
		viewerScrollBar.setBlockIncrement(visAmount);
		viewerScrollBar.setMaximum(amount);	
		viewerScrollBar.setValue(value);
		viewerScrollBar.addAdjustmentListener(this);
	}


}
