package mpi.eudico.client.annotator.interlinear.edit;

import java.awt.Component;
import java.util.ArrayList;
import java.util.List;
import mpi.eudico.client.annotator.commands.AnnotationsFromSuggestionSetCommand;
import mpi.eudico.client.annotator.commands.Command;
import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSelectionEvent;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSelectionListener;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSetEvent;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSetListener;
import mpi.eudico.client.annotator.interlinear.edit.event.SuggestionSetProvider;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionModel;
import mpi.eudico.client.annotator.interlinear.edit.model.IGTSuggestionViewerModel;
import mpi.eudico.client.annotator.layout.InterlinearizationManager;
import mpi.eudico.server.corpora.clom.Annotation;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.server.corpora.clomimpl.type.Constraint;
import mpi.eudico.util.TimeRelation;
import nl.mpi.lexan.annotyzers.Mediator;
import nl.mpi.lexan.annotyzers.helpers.Information;
import nl.mpi.lexan.annotyzers.helpers.Position;
import nl.mpi.lexan.annotyzers.helpers.Suggestion;
import nl.mpi.lexan.annotyzers.helpers.SuggestionSet;
import nl.mpi.lexan.annotyzers.helpers.prompt.Prompt;

// Maven without test -DskipTests=true
/**
 * A mediator class that initializes and maintains the communication with 
 * a framework of parsers and recognizers (Lexan). This framework communicates 
 * with both LEXUS and ELAN.
 *  
 * @author Han Sloetjes
 *
 */
public class InterlinearMediator implements Mediator, SuggestionSelectionListener, SuggestionSetProvider {
	private InterlinearizationManager manager;
	private TranscriptionImpl transcription;
	private Mediator lexanMediator;
	private List<SuggestionSetListener> suggestionListeners;
	
	// temp
	private List<AnnotConfig> configs;
	private boolean recursive = false;
	
	private boolean sugWindowOpened = false;
	
	/**
	 * 
	 */
	public InterlinearMediator(InterlinearizationManager manager) {
		super();
		this.manager = manager;
		this.transcription = (TranscriptionImpl) manager.getTranscription();
		configs = new ArrayList<AnnotConfig>(6);
		suggestionListeners = new ArrayList<SuggestionSetListener>(4);
	}	
	
	public void setMediator(Mediator lexanMediator) {
		this.lexanMediator = lexanMediator;
	}
	
	public Mediator getConnectedMediator() {
		return lexanMediator;
	}

	public TranscriptionImpl getTranscription() {
		return transcription;
	}
	
	/**
	 * Adds a configuration to the list of tier-annotyzer configurations
	 *  
	 * @param config
	 */
	public void addConfig(AnnotConfig config) {
		if (!configs.contains(config)) {
			configs.add(config);
			
			List<AnnotConfig> confList = new ArrayList<AnnotConfig>();			
			if(config.isTypeMode()){
				confList.addAll(((AnnotTypeConfig)config).getTierConfigurations());
			} else {
				confList.add(config);
			}
			
			List<List<String>> allSources = new ArrayList<List<String>>(1);
			List<List<String>> allDests = new ArrayList<List<String>>(1);
			
			for(AnnotConfig ac : confList){
				List<String> src = new ArrayList<String>(1);
				src.add(ac.getSource());
				
				
				List<String> dest = new ArrayList<String>(1);
				dest.addAll(ac.getDest());
				
				allSources.add(src);				
				allDests.add(dest);
			}
			
			
			boolean success = initAnnotyzer(allSources, 
					allDests, config.getAnnotId());
// 			boolean success = initAnnotyzer(config.getSrcTier(), 
//					config.getDestTier(), config.getAnnotId());
			
			System.out.println("Initialized annotation analyzer: " + success);
		}
		// notify the editor
		manager.getInterEditor().configsChanged();
	}
	
	public void removeConfig(AnnotConfig config) {
		if (configs.contains(config)) {
			configs.remove(config);
			
			List<AnnotConfig> confList = new ArrayList<AnnotConfig>();			
			if(config.isTypeMode()){
				confList.addAll(((AnnotTypeConfig)config).getTierConfigurations());
			} else {
				confList.add(config);
			}
			
			if(confList.size() == 0){
				return;
			}
			
			List<List<String>> allSources = new ArrayList<List<String>>(1);
			List<List<String>> allDests = new ArrayList<List<String>>(1);
			
			for(AnnotConfig ac : confList){
				List<String> src = new ArrayList<String>(1);
				src.add(ac.getSource());
				
				
				List<String> dest = new ArrayList<String>(1);
				dest.addAll(ac.getDest());
				
				allSources.add(src);				
				allDests.add(dest);
			}
			
			unloadAnnotyzer(allSources, allDests, config.getAnnotId());
		}
	}
	
	/**
	 * Gives access to the list of configurations
	 * @return
	 */
	public List<AnnotConfig> getConfigurations() {
		return configs;
	}
	
	/**
	 * Add the listener to the list.
	 * 
	 * @param listener the suggestion set listener
	 */
	public void addSuggestionSetListener(SuggestionSetListener listener) {
		if (listener != null && !suggestionListeners.contains(listener)) {
			suggestionListeners.add(listener);
		}
	}
	
	/**
	 * Removes the specified listener from the list of listeners.
	 * 
	 * @param listener the listener to remove
	 */
	public void removeSuggestionSetListener(SuggestionSetListener listener) {
		suggestionListeners.remove(listener);
	}
	
	/**
	 * Implementation of SuggestionSelectionListener.
	 * Notification of selection of a suggestion set.
	 * 
	 * @param event the suggestion selection event. The source of an event is a suggestion viewer model
	 */
	@Override
	public void suggestionSelected(SuggestionSelectionEvent event) {
		if (event != null) {
			IGTSuggestionViewerModel viewModel = (IGTSuggestionViewerModel) event.getSource();
			int row = event.getSelectedRow();
			try {
				IGTSuggestionModel sugModel = viewModel.getRowData(row);
				SuggestionSet sugSet = sugModel.getSuggestionSet();
				
				// create annotations
				AnnotationsFromSuggestionSetCommand command = new AnnotationsFromSuggestionSetCommand("Annotations from Suggestions");
				command.execute(transcription, new Object[]{sugSet});
				
			} catch (ArrayIndexOutOfBoundsException abe) {
				// TODO show message
			}
		}		
	}
	
	/**
	 * Sets the recursive behavior flag.
	 * Temporary.
	 * @param recurs flag for recursive processing
	 */
	public void setRecursive(boolean recurs) {
		this.recursive = recurs;
	}

// ############ ELAN side of mediator interface #########################
	
	public boolean changeAnnotation(Position pos, Suggestion ann) {
		if (pos == null) {
			// warn
			System.out.println("Change annotation: Position is null");
			return false;
		}
		if (ann == null) {
			// warn
			System.out.println("Change annotation: no annotation specified");
			return false;
		}
		
		boolean changed = false;
		TierImpl destTier = (TierImpl) transcription.getTierWithId(pos.tierId);
		
		if (destTier != null) {
			long t = (pos.beginTime + pos.endTime) / 2;
			AbstractAnnotation aa = (AbstractAnnotation) destTier.getAnnotationAtTime(t);
			if (aa != null) {
				if (ann.content != null) {
					aa.setValue(ann.content);
				} else {
					aa.setValue("");
				}
				changed = true;
			} else {
				// warn
				System.out.println("Change annotation: there is no annotation at the specified position");
			}
		}
		return changed;
	}

	/**
	 * Creates one new annotation at the specified position.
	 * In case of a subdivision tier the annotation will be added after the last annotation
	 * in the interval (if it exists).
	 */
	public boolean newAnnotation(Position pos, Suggestion ann) {
		if (pos == null) {
			// warn
			System.out.println("New annotation: Position is null");
			return false;
		}
		if (ann == null) {
			// warn
			System.out.println("New annotation: no new annotation specified");
			return false;
		}
		
		boolean created = false;
		TierImpl destTier = (TierImpl) transcription.getTierWithId(pos.tierId);
		
		if (destTier != null) {
			Constraint cons = destTier.getLinguisticType().getConstraints();
			if (cons == null) {
				// warn, illegal tier type for destination?
				return false;
			}
			AbstractAnnotation aa = (AbstractAnnotation) destTier.getAnnotationAtTime(pos.endTime - 1);
			if (aa != null) { // remove it first
				destTier.removeAnnotation(aa);
				/*
				if (cons.getStereoType() == Constraint.SYMBOLIC_ASSOCIATION) {
					// warn, there is already an annotation
					System.out.println("New annotation: there is already an annotation at this Position");
				} else if (cons.supportsInsertion()) {
					AbstractAnnotation na = (AbstractAnnotation) destTier.createAnnotationAfter(aa);
					if (na != null) {
						na.setValue(ann.content);
						created = true;
						if (recursive) {
							Position nextPos = new Position();
							nextPos.tierId = pos.tierId;
							nextPos.beginTime = na.getBeginTimeBoundary();
							nextPos.endTime = na.getEndTimeBoundary();
							annotyze(nextPos);
						}
					}
				}
				*/
			} //else { // no annotation yet
			
			long t = (pos.beginTime + pos.endTime) / 2;
			AbstractAnnotation na = (AbstractAnnotation) destTier.createAnnotation(t, t);
			if (na != null) {
				na.setValue(ann.content);
				created = true;
				if (recursive) {
					annotyze(pos);
				}
			}				
			//}
		}
		
		return created;
	}

	/**
	 * Creates multiple new annotations at the specified position.
	 * In case of a subdivision tier the annotations will be added after the last annotation
	 * in the interval (if it exists).
	 */
	public boolean newAnnotation(Position pos, List<Suggestion> annotations) {
		if (pos == null) {
			// warn
			System.out.println("New annotation: Position is null");
			return false;
		}
		if (annotations == null || annotations.size() == 0) {
			// warn
			System.out.println("New annotation: no new annotations specified");
			return false;
		}
		boolean created = false;
		TierImpl destTier = (TierImpl) transcription.getTierWithId(pos.tierId);
		
		if (destTier != null) {
			Constraint cons = destTier.getLinguisticType().getConstraints();
			if (cons == null) {
				// warn, illegal tier type for destination?
				return false;
			}
			
			List<AbstractAnnotation> recurses = null;
			if (recursive) {
				recurses = new ArrayList<AbstractAnnotation>();
			}
			
			List allAnns = destTier.getAnnotations();
			List<Annotation> removables = new ArrayList<Annotation>();
			AbstractAnnotation ra = null;
			for (int i = 0; i < allAnns.size(); i++) {
				ra = (AbstractAnnotation) allAnns.get(i);
				if (TimeRelation.overlaps(ra, pos.beginTime, pos.endTime)) {
					removables.add(ra);
				}
				if (ra.getBeginTimeBoundary() >= pos.endTime) {
					break;
				}
			}
			
			if (removables.size() > 0) {
				for (Annotation ann : removables) {
					destTier.removeAnnotation(ann);
				}
			}
			
			//AbstractAnnotation aa = (AbstractAnnotation) destTier.getAnnotationAtTime(pos.endTime - 1);
			
			//if (aa != null) {// delete it first
			//	destTier.removeAnnotation(aa);
				/*
				if (cons.getStereoType() == Constraint.SYMBOLIC_ASSOCIATION) {
					// warn, there is already an annotation
					System.out.println("New annotation: there is already an annotation at this Position");
				} else if (cons.supportsInsertion()) {
					// add to the end of the chain of annotations
					// all temporary code, should use undoable commands here
					AbstractAnnotation na;
					for (Suggestion ann : annotations) {
						na = (AbstractAnnotation) destTier.createAnnotationAfter(aa);
						if (na != null) {
							na.setValue(ann.content);
							created = true;
							aa = na;
							if (recursive) {
								recurses.add(na);
							}
						}
					}
				} else {
					// cannot create multiple annotations on an included in tier, if there is already an annotation
				}
				*/
			//} //else {
			
			AbstractAnnotation na = null;
			AbstractAnnotation aa = null;
			Suggestion ai;
			for (int i = 0; i < annotations.size(); i++) {
				ai = annotations.get(i);
				
				if (i == 0) {					
					if(cons.getStereoType() == Constraint.INCLUDED_IN || cons.getStereoType() == Constraint.TIME_SUBDIVISION){
//						long dur = (pos.endTime - pos.beginTime) / annotations.size();
//						na = (AbstractAnnotation) destTier.createAnnotation(pos.beginTime + ( i * dur), pos.beginTime + ( (i+1) * dur));
						na = (AbstractAnnotation) destTier.createAnnotation(pos.beginTime, pos.endTime);
					} else {
						long t = (pos.beginTime + pos.endTime) / 2;
						na = (AbstractAnnotation) destTier.createAnnotation(t, t);
					}
					
					if (na != null) {
						na.setValue(ai.content);
						aa = na;
						created = true;
						if (recursive) {
							recurses.add(na);
						}
					}
					if (cons.getStereoType() == Constraint.SYMBOLIC_ASSOCIATION) {
						break;
					}
				} else {
					if (aa != null) {
						na = (AbstractAnnotation) destTier.createAnnotationAfter(aa);
						if (na != null) {
							na.setValue(ai.content);
							aa = na;
							created = true;
							if (recursive) {
								recurses.add(na);
							}
						}
					}
				}
			}
				
			//}
			
			if (recurses != null && recurses.size() > 0) {
				for (AbstractAnnotation aann : recurses) {
					Position np = new Position();
					np.tierId = aann.getTier().getName();
					np.beginTime = aann.getBeginTimeBoundary();
					np.endTime = aann.getEndTimeBoundary();
					annotyze(np);
				}
			}
		}
		
		
		return created;
	}
	
	/**
	 * If a list of suggestions is produced by one of the extensions, notify listeners.
	 * For now this mechanism is used to push the suggestions to the user interface layer.
	 * 
	 * @param sugSets the list of suggestion sets. Each set represents a list of possible 
	 * values from which the user can pick one
	 */
	public boolean newAnnotations(List<SuggestionSet> sugSets) {
		if (sugSets != null) {
			if(!sugWindowOpened ){
				 // hier add transcription or retrieve root tier?
				SuggestionSetEvent event = new SuggestionSetEvent(this, sugSets);
				
				for (SuggestionSetListener ssl : suggestionListeners) {
					ssl.suggestionSetDelivered(event);
				}
				sugWindowOpened = true;
				return true;	
			}
		}
		return false;
	}
	
	public void interLinearizeActionCompleted(){
		sugWindowOpened = false;
	}

	public boolean newTier(String arg0) {
		// TODO Auto-generated method stub
		return false;
	}

	public String prompt(Prompt arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Returns the annotations from the specified position 
	 * (i.e. from the specified tier within the specified interval). 
	 */
	public List<Suggestion> readAnnotation(Position pos) {
		if (pos == null) {
			// warn
			System.out.println("Read annotations: Position is null");
			return null;
		}
		List<Suggestion> annotations = new ArrayList<Suggestion>();
		
		TierImpl destTier = (TierImpl) transcription.getTierWithId(pos.tierId);
		
		if (destTier != null) {
			List curAnnos = destTier.getAnnotations();
			AbstractAnnotation aa = null;
			for (int i = 0; i < curAnnos.size(); i++) {
				aa = (AbstractAnnotation) curAnnos.get(i);
				if (aa.getEndTimeBoundary() < pos.beginTime) {
					continue;
				}
				if (aa.getBeginTimeBoundary() > pos.endTime) {
					break;
				}
				if (aa.getBeginTimeBoundary() >= pos.beginTime && aa.getEndTimeBoundary() <= pos.endTime) {
					Suggestion a = new Suggestion();
					a.content = aa.getValue();
					a.position = pos;//??
					annotations.add(a);
				}
			}
		}
		
		return annotations;
	}

	public boolean remAnnotation(Position pos) {
		if (pos == null) {
			// warn
			System.out.println("Remove annotation: Position is null");
			return false;
		}
		
		TierImpl destTier = (TierImpl) transcription.getTierWithId(pos.tierId);
		
		if (destTier != null) {
			long t = (pos.beginTime + pos.endTime) / 2;
			AbstractAnnotation aa = (AbstractAnnotation) destTier.getAnnotationAtTime(t);
			if (aa != null) {
				// use the undoable delete command
				Command command = ELANCommandFactory.createCommand(transcription, 
						ELANCommandFactory.DELETE_ANNOTATION);
				command.execute(destTier, new Object[]{ELANCommandFactory.getViewerManager(transcription), aa});
				//destTier.removeAnnotation(aa);
				// notify the annotyzer via the mediator of the removal? 
				//should there be an annotationDeleted metod in the api? 
				//lexanMediator.remAnnotation(pos);
				return true;
			} else {
				// warn
				System.out.println("Remove annotation: there is no annotation at the specified position");
			}
		}

		return false;
	}

	public boolean remTier(String arg0) {
		// TODO Auto-generated method stub
		return false;
	}

// ##################### LEXAN side of mediator interface ###################
	public void tierDeleted(String arg0) {
		lexanMediator.tierDeleted(arg0);		
	}

	public boolean initAnnotyzer(List<List<String>> arg0,
			List<List<String>> arg1, Information arg2) {
		return lexanMediator.initAnnotyzer(arg0, arg1, arg2);
	}

	public List<Information> listAnnotyzers() {
		return lexanMediator.listAnnotyzers();
	}

	public void annotyze(Position pos) {
		lexanMediator.annotyze(pos);
	}

//	public void annotyze(Range arg0) {
//		lexanMediator.annotyze(arg0);
//	}

	public boolean change(Position pos) {
		return lexanMediator.change(pos);
	}


	public void initAnnotyzers(List<String> arg0) {
		lexanMediator.initAnnotyzers(arg0);
	}
	
	public boolean unloadAnnotyzer(List<List<String>> arg0,
			List<List<String>> arg1, Information arg2) {
		return lexanMediator.unloadAnnotyzer(arg0, arg1, arg2);
	}

	/**
	 * Tells the mediator to load stored information from the specified 
	 * location.
	 * 
	 * @param folderPath the location from where to load state information
	 * @return a success flag
	 */
	@Override
	public boolean loadState(String folderPath) {
		return lexanMediator.loadState(folderPath);
	}

	/**
	 * Informs the lexan component(s) to store current state in the 
	 * specified folder.
	 * 
	 * @param folderPath the path to a folder in which annotyzers should store their state
	 * @return a success flag
	 */
	@Override
	public boolean saveState(String folderPath) {
		return lexanMediator.saveState(folderPath);
	}

	@Override
	public Component getConfigurationComponent(Information annotyzer) {
		// TODO Auto-generated method stub
		return lexanMediator.getConfigurationComponent(annotyzer);
	}
	
}
