package mpi.eudico.client.annotator.prefs.gui;

import mpi.eudico.client.annotator.ElanLocale;
import mpi.eudico.client.annotator.Preferences;

import mpi.eudico.client.annotator.prefs.PreferenceEditor;

import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;


/**
 * A panel for changing UI related settings.
 * 
 * @author Mark Blokpoel
  */
public class UIPrefsPanel extends AbstractEditPrefsPanel implements PreferenceEditor {
    private JComboBox nrOfRecentItemsCBox;
    private Integer origNrRecentItems = 5;
    private JCheckBox tooltipCB;
    private boolean origToolTipEnabled = true;
    private JCheckBox showAnnotationCountCB;
    private boolean origShowAnnotationCount = false; 
    private JRadioButton useBufferedPaintingRB;
    private JRadioButton useDirectPaintingRB;
    /**
     * 03-2013 user interface for buffered painting setting that used to be only a 
     * command line parameter. Default is now false.
     */
    private boolean origUseBufferedPainting = false;// default false

    /**
     * Creates a new PlatformPanel instance
     */
    public UIPrefsPanel() {
        super(ElanLocale.getString("PreferencesDialog.Category.UI"));
        readPrefs();
        initComponents();
    }

    private void readPrefs() {
        Object val = Preferences.get("UI.RecentItems", null);

        if (val instanceof Integer) {
            origNrRecentItems = (Integer) val;
        }
        
        val = Preferences.get("UI.ToolTips.Enabled", null);
        
        if (val instanceof Boolean) {
        	origToolTipEnabled = (Boolean) val;
        }
        
        val = Preferences.get("UI.MenuItems.ShowAnnotationCount", null);
    	if (val instanceof Boolean)
    		origShowAnnotationCount = (Boolean) val;
    	
    	val = Preferences.get("UI.UseBufferedPainting", null);
    	if (val instanceof Boolean) {
    		origUseBufferedPainting = (Boolean) val;
    	}
    }

    private void initComponents() {  
    	GridBagConstraints gbc;
    	Font plainFont;
    	
    	//recent items panel
    	Integer[] nrOfRecentItemsList = { 5, 10, 15, 20, 25, 30 };
        nrOfRecentItemsCBox = new JComboBox(nrOfRecentItemsList);
        nrOfRecentItemsCBox.setSelectedItem(origNrRecentItems);
        plainFont = nrOfRecentItemsCBox.getFont().deriveFont(Font.PLAIN);
        nrOfRecentItemsCBox.setFont(plainFont);
        
        JLabel recentItemsLabel = new JLabel(ElanLocale.getString("PreferencesDialog.UI.RecentItems"));
        recentItemsLabel.setFont(plainFont);

        JPanel recentItemsPanel = new JPanel(new GridBagLayout());
        gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;        
        gbc.insets = topInset;
        recentItemsPanel.add(recentItemsLabel,gbc);

        gbc.gridx = 1;
        gbc.insets = leftInset;
        recentItemsPanel.add(nrOfRecentItemsCBox, gbc);
    	
        gbc.gridx = 2;
        gbc.fill = GridBagConstraints.HORIZONTAL;  
        gbc.weightx = 1.0;
        recentItemsPanel.add(new JPanel(), gbc); // filler
        
        // main panel    	
    	int gy=0;
    	       
        gbc = new GridBagConstraints();
        gbc.anchor = GridBagConstraints.NORTHWEST;
        gbc.fill = GridBagConstraints.HORIZONTAL; 
        gbc.insets = globalPanelInset;
        gbc.gridy = gy++;
        outerPanel.add(recentItemsPanel, gbc);
              
        /*
        JLabel relaunchLabel = new JLabel();
        ImageIcon relaunchIcon = null;

        // add relaunch icon
        try {
            relaunchIcon = new ImageIcon(this.getClass()
                                             .getResource("/toolbarButtonGraphics/general/Refresh16.gif"));
            relaunchLabel.setIcon(relaunchIcon);
        } catch (Exception ex) {
            relaunchLabel.setText(ElanLocale.getString(
                    "PreferencesDialog.Relaunch"));
        }

        relaunchLabel.setToolTipText(ElanLocale.getString(
                "PreferencesDialog.Relaunch.Tooltip"));
        nrOfRecentItemsCBox.setToolTipText(ElanLocale.getString(
                "PreferencesDialog.Relaunch.Tooltip"));

        gbc.gridx = 1;
        gbc.gridwidth = 1;
        gbc.fill = GridBagConstraints.NONE;
        gbc.anchor = GridBagConstraints.EAST;
        gbc.weightx = 0.0;
        add(relaunchLabel, gbc);
        */
        
        gbc.gridy = gy++;
        gbc.insets = catInset;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        outerPanel.add(new JLabel(ElanLocale.getString("PreferencesDialog.UI.ToolTip")), gbc);
        
        tooltipCB = new JCheckBox(ElanLocale.getString("PreferencesDialog.UI.ToolTip.Enabled"));
        tooltipCB.setSelected(origToolTipEnabled);
        tooltipCB.setFont(tooltipCB.getFont().deriveFont(Font.PLAIN));
       
        gbc.gridy = gy++;
        gbc.insets = globalInset;
        outerPanel.add(tooltipCB, gbc);
        
        gbc.gridy = gy++;
        gbc.insets = catInset;
        outerPanel.add(new JLabel(ElanLocale.getString("PreferencesDialog.UI.MenuOptions")), gbc);
        
        showAnnotationCountCB = new JCheckBox(ElanLocale.getString("PreferencesDialog.UI.MenuOptions.ShowAnnotationCount"));
        showAnnotationCountCB.setSelected(origShowAnnotationCount);
        showAnnotationCountCB.setFont(showAnnotationCountCB.getFont().deriveFont(Font.PLAIN));
        gbc.gridy = gy++;
        gbc.insets = globalInset;
        outerPanel.add(showAnnotationCountCB, gbc);
        
        // hier add label?
        gbc.gridy = gy++;
        gbc.insets = catInset;
        outerPanel.add(new JLabel(ElanLocale.getString("PreferencesDialog.UI.PaintingStrategy")), gbc);
        
        ButtonGroup bGroup = new ButtonGroup();
        useBufferedPaintingRB = new JRadioButton(ElanLocale.getString("PreferencesDialog.UI.UseBufferedPainting"));
        useBufferedPaintingRB.setSelected(origUseBufferedPainting);
        useBufferedPaintingRB.setFont(tooltipCB.getFont().deriveFont(Font.PLAIN));
        useDirectPaintingRB = new JRadioButton(ElanLocale.getString("PreferencesDialog.UI.UseDirectPainting"));
        useDirectPaintingRB.setSelected(!origUseBufferedPainting);
        useDirectPaintingRB.setFont(tooltipCB.getFont().deriveFont(Font.PLAIN));
        bGroup.add(useBufferedPaintingRB);
        bGroup.add(useDirectPaintingRB);
       
        // add buttons        
        gbc.gridy = gy++;
        gbc.insets = globalInset;
        outerPanel.add(useBufferedPaintingRB, gbc);
      
        gbc.gridy = gy++;
        outerPanel.add(useDirectPaintingRB, gbc);
        
        gbc.gridy = gy++;       
        gbc.fill = GridBagConstraints.BOTH;
        gbc.weighty = 1.0;
        gbc.weightx = 1.0;        
        outerPanel.add(new JPanel(), gbc); // filler
    }
    
    /**
     * Returns
     *
     * @return a map containing the changes
     */
    public Map getChangedPreferences() {
        if (isChanged()) {
            Map<String, Object> chMap = new HashMap<String, Object>(6);

            if (nrOfRecentItemsCBox.getSelectedItem() != origNrRecentItems) {
                chMap.put("UI.RecentItems",
                    nrOfRecentItemsCBox.getSelectedItem());
            }
            // will be handled by ElanLayoutManager (arbitrary choice)
            if (tooltipCB.isSelected() != origToolTipEnabled) {
            	chMap.put("UI.ToolTips.Enabled",
            			Boolean.valueOf(tooltipCB.isSelected()));
            }
            if(showAnnotationCountCB.isSelected() != origShowAnnotationCount){
        		chMap.put( "UI.MenuItems.ShowAnnotationCount", Boolean.valueOf(showAnnotationCountCB.isSelected()) );
        	}
            
            if (useBufferedPaintingRB.isSelected() != origUseBufferedPainting) {
            	chMap.put("UI.UseBufferedPainting", Boolean.valueOf(useBufferedPaintingRB.isSelected()));
            }
            return chMap;
        }

        return null;
    }

    /**
     * Returns whether any of the settings has changed
     *
     * @return whether anything changed
     */
    public boolean isChanged() {
        if (nrOfRecentItemsCBox.getSelectedItem() != origNrRecentItems || 
        		tooltipCB.isSelected() != origToolTipEnabled || 
        		showAnnotationCountCB.isSelected() != origShowAnnotationCount ||
        		origUseBufferedPainting != useBufferedPaintingRB.isSelected()) {
            return true;
        }

        return false;
    }
}
