package mpi.eudico.client.annotator.webserviceclient.weblicht;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import mpi.eudico.client.annotator.gui.multistep.MultiStepPane;
import mpi.eudico.client.annotator.gui.multistep.StepPane;

/**
 * A step in the interaction with WebLicht.
 * 
 * @author Han Sloetjes
 */
@SuppressWarnings("serial")
public class WebLichtStep2 extends StepPane implements DocumentListener {
	private JTextArea textArea;
	private JLabel textLabel;
	//private JButton uploadButton;

	public WebLichtStep2(MultiStepPane multiPane) {
		super(multiPane);
		
		initComponents();
	}

    /**
     * Initialize the panel.
     * 
     * @see mpi.eudico.client.annotator.gui.multistep.StepPane#initComponents()
     */
    public void initComponents() {
    	setLayout(new GridBagLayout());
    	setBorder(new EmptyBorder(5, 10, 5, 10));
    	
    	textLabel = new JLabel("Type or paste text into the text field");
    	textArea = new JTextArea();
    	textArea.setWrapStyleWord(true);
    	textArea.setLineWrap(true);
    	//uploadButton = new JButton("Upload");
    	//uploadButton.addActionListener(this);
    	
    	GridBagConstraints gbc = new GridBagConstraints();
    	gbc.insets = new Insets(6, 0, 6, 0);
    	gbc.anchor = GridBagConstraints.NORTHWEST;
    	add(textLabel, gbc);
    	
    	gbc.gridy = 1;
    	gbc.insets = new Insets(2, 0, 2, 0);
    	gbc.fill = GridBagConstraints.BOTH;
    	gbc.weightx = 1.0;
    	gbc.weighty = 1.0;
    	add(new JScrollPane(textArea), gbc);
    	
//    	gbc.gridy = 2;
//    	gbc.fill = GridBagConstraints.NONE;
//    	gbc.weightx = 0.0;
//    	gbc.weighty = 0.0;
//    	add(uploadButton, gbc);
    	
    	textArea.getDocument().addDocumentListener(this);
    }

	/**
	 * Stores the text in the pane's properties.
	 */
	public boolean leaveStepForward() {
		String text = textArea.getText(); 
		if (text != null && text.length() > 0) {
			multiPane.putStepProperty("InputText", text);
			return true;
		} else {
			// popup a info message
			return false;
		}
	}

	
	@Override
	public void enterStepBackward() {
		checkText();
	}

	public String getStepTitle() {
		return "WebLicht text upload";
	}

	/**
	 * Document listener
	 */
	public void changedUpdate(DocumentEvent arg0) {
		checkText();
		
	}

	/**
	 * Document listener
	 */
	public void insertUpdate(DocumentEvent arg0) {
		checkText();
		
	}

	/**
	 * Document listener
	 */
	public void removeUpdate(DocumentEvent arg0) {
		checkText();
		
	}
	
	/**
	 * Checks whether there is any text at all and updates the next button status. 
	 */
	private void checkText() {
		String t = textArea.getText();
		if (t != null && t.length() > 0) {
			multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, true);
		} else {
			multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, false);
		}
	}
}
