package mpi.eudico.client.annotator.webserviceclient.weblicht;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import mpi.eudico.client.annotator.gui.multistep.MultiStepPane;
import mpi.eudico.client.annotator.gui.multistep.StepPane;
/**
 * Provides means to specify what type of processing to apply, 
 * what tools to call.
 * 
 * @author Han Sloetjes
 */
public class WebLichtStep3 extends StepPane implements ChangeListener {
	private JLabel sentenceDurationLabel;
	private JTextField sentenceDurationTF;
	
	private JCheckBox tokenizeCB;
	private JCheckBox posTagsCB;
	private JCheckBox lemmaCB;
	
	public String getStepTitle() {
		return "Configure the processing chain for the text";
	}

	public WebLichtStep3(MultiStepPane multiPane) {
		super(multiPane);
		
		initComponents();
	}

	/**
	 * Populates the panel with elements for selecting the services to call.
	 */
	protected void initComponents() {
		super.initComponents();
		JPanel optionPanel = new JPanel(new GridBagLayout());
		optionPanel.setBorder(new TitledBorder("Options"));
		
		sentenceDurationLabel = new JLabel ("Specify the duration per sentence");
		sentenceDurationTF = new JTextField(12);
		sentenceDurationTF.setText("3000");
		tokenizeCB = new JCheckBox("Tokenize sentences", true);
		posTagsCB = new JCheckBox("Add Part of Speech tags", false);
		lemmaCB = new JCheckBox("Add Lemma tags", false);
		
    	setLayout(new GridBagLayout());
    	Insets insets = new Insets(2, 0, 2, 0);
    	Insets globalInsets = new Insets(5, 10, 5, 10);
    	//setBorder(new EmptyBorder(12, 12, 12, 12));
    	GridBagConstraints gbc = new GridBagConstraints();
    	gbc.insets = insets;
    	gbc.anchor = GridBagConstraints.NORTHWEST;
    	gbc.fill = GridBagConstraints.HORIZONTAL;
    	gbc.weightx = 1.0;
    	gbc.gridwidth = 2;

    	optionPanel.add(tokenizeCB, gbc);
    	
    	gbc.gridy = 1;
    	optionPanel.add(posTagsCB, gbc);
    	
    	gbc.gridy = 2;
    	optionPanel.add(lemmaCB, gbc);
    	
    	gbc.gridy = 3;
    	gbc.gridwidth = 1;
    	gbc.insets = new Insets(12, 0, 2, 0);
    	gbc.weightx = 0.0;
    	gbc.fill = GridBagConstraints.NONE;
    	gbc.anchor = GridBagConstraints.WEST;
    	optionPanel.add(sentenceDurationLabel, gbc);
    	
    	gbc.gridx = 1;
    	gbc.insets = new Insets(12, 4, 2, 0);;
    	optionPanel.add(sentenceDurationTF, gbc);
    	
    	gbc.gridy = 4;
    	gbc.gridx = 0;
    	gbc.gridwidth = 2;
    	gbc.fill = GridBagConstraints.BOTH;
    	gbc.weightx = 1.0;
    	gbc.weighty = 1.0;
    	optionPanel.add(new JPanel(), gbc);
    	
    	gbc = new GridBagConstraints();
    	gbc.fill = GridBagConstraints.BOTH;
    	gbc.weightx = 1.0;
    	gbc.weighty = 1.0;
    	gbc.insets = globalInsets;
    	add(optionPanel, gbc);
    	tokenizeCB.addChangeListener(this);
    	
	}

	/**
	 * Updates elements based on selected state of the tokenize checkbox
	 */
	public void stateChanged(ChangeEvent event) {
		if (event.getSource() == tokenizeCB) {
			boolean selected = tokenizeCB.isSelected();
			posTagsCB.setEnabled(selected);
			lemmaCB.setEnabled(selected);
		}
	}

	@Override
	public void enterStepForward() {
		multiPane.setButtonEnabled(MultiStepPane.NEXT_BUTTON, true);
	}

	@Override
	public boolean leaveStepForward() {
		boolean tokenize = tokenizeCB.isSelected();
		if (tokenize) {
			multiPane.putStepProperty("IncludeTokenize", true);
			multiPane.putStepProperty("IncludePOS", posTagsCB.isSelected());
			multiPane.putStepProperty("IncludeLemma", lemmaCB.isSelected());
		} else {
			multiPane.putStepProperty("IncludeTokenize", false);
			multiPane.putStepProperty("IncludePOS", false);
			multiPane.putStepProperty("IncludeLemma", false);
		}
		int duration = 3000;
		String durInput = sentenceDurationTF.getText();
		if (durInput != null) {
			try {
				duration = Integer.parseInt(durInput);
			} catch (NumberFormatException nfe) {
				
			}
		}
		multiPane.putStepProperty("SentenceDuration", duration);
		
		return true;
	}

	
}
