package mpi.eudico.client.annotator.webserviceclient.weblicht;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.border.EmptyBorder;

import org.xml.sax.SAXException;

import mpi.eudico.client.annotator.commands.ELANCommandFactory;
import mpi.eudico.client.annotator.commands.MergeTranscriptionsByAddingCommand;
import mpi.eudico.client.annotator.gui.multistep.MultiStepPane;
import mpi.eudico.client.annotator.gui.multistep.StepPane;
import mpi.eudico.client.annotator.util.ClientLogger;
import mpi.eudico.server.corpora.clom.Tier;
import mpi.eudico.server.corpora.clomimpl.abstr.AbstractAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.AlignableAnnotation;
import mpi.eudico.server.corpora.clomimpl.abstr.TierImpl;
import mpi.eudico.server.corpora.clomimpl.abstr.TranscriptionImpl;
import mpi.eudico.webserviceclient.weblicht.TCFConstants;
import mpi.eudico.webserviceclient.weblicht.TCFtoTranscription;
import mpi.eudico.webserviceclient.weblicht.TiersToTCF;
import mpi.eudico.webserviceclient.weblicht.WebLichtWsClient;

/**
 * Converts the contents of a tier to tcf (or to text that will be sent to weblicht to convert to tcf)?
 * 
 * 
 * @author Han Sloetjes
 *
 */
public class WebLichtTierBasedStep4 extends StepPane {
	private JLabel progressLabel;
	private JProgressBar progressBar;
	private boolean cancelled = false;
	private String tierName;
	private String contentType;
	private TranscriptionImpl trans;
	private TiersToTCF tiersToTCF;
	
	
	public WebLichtTierBasedStep4(MultiStepPane multiPane) {
		super(multiPane);
		
		initComponents();
	}

	/**
	 * Show a progress bar.
	 */
	protected void initComponents() {
		setLayout(new GridBagLayout());
		setBorder(new EmptyBorder(5, 10, 5, 10));
		
		progressLabel = new JLabel("Processing tier... please wait");
		progressBar = new JProgressBar(0, 100);
		
		Insets insets = new Insets(2, 0, 2, 0);
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.anchor = GridBagConstraints.WEST;
		gbc.insets = insets;
		gbc.fill = GridBagConstraints.HORIZONTAL;
		gbc.weightx = 1.0;
		
		add(progressLabel, gbc);
		gbc.gridy = 1;
		add(progressBar, gbc);
	}

	@Override
	public void enterStepForward() {
		// everything has been set here. Start processing
		multiPane.setButtonEnabled(MultiStepPane.ALL_BUTTONS, false);
		
		tierName = (String) multiPane.getStepProperty("Tier");
		contentType = (String) multiPane.getStepProperty("ContentType");
		trans = (TranscriptionImpl) multiPane.getStepProperty("transcription");
		
		// can store /retrieve a selected web service
		// can allow an id for the content language 
		
		WLTThread wltThread = new WLTThread();
		wltThread.start();	
		
		multiPane.setButtonEnabled(MultiStepPane.CANCEL_BUTTON, true);
	}
	
	private class WLTThread extends Thread {

		/**
		 * The actual processing:
		 * - convert annotations to tcf
		 * - upload to selected service(s)
		 * - convert result tcf to transcription
		 * - update times to match the original annotation times 
		 * - merge transcription
		 */
		public void run() {

			if (trans != null) {
				TierImpl tier = (TierImpl) trans.getTierWithId(tierName);
				tiersToTCF = new TiersToTCF();
				WebLichtWsClient wsClient = new WebLichtWsClient();
				String tcfString = null;
				
				if (contentType.equals("Sentence")) {// should be an enum type rather than a String
					processSentenceLevel(tier);
					return;
				} else if (contentType.startsWith("Word")) {
					// get the parent tier
					if (tier.hasParentTier()) {
						tcfString = tiersToTCF.toTCFString(tier, "Token");
						System.out.println("TCF:");
						System.out.println(tcfString);
					} else {
						// this is an error. Treat all annotations as words in one sentence?
					}
				}
				
				if (tcfString != null) {
					
					String tcfString2 = null;
					// check what service to call, based on user input or based on content type of the source tier?
					//wsClient.callWithTCF(toolUrl, tcfString);
					if (contentType.equals("Sentence")) {// always tokenize, no sense not to do so
						String sentenceTokenUrl = "service-opennlp/annotate/tok-sentences";
						String posTagUrl = "service-opennlp/annotate/postag";
						
						tcfString2 = wsClient.callWithTCF(posTagUrl, tcfString);
						System.out.println("TCF2:");
						System.out.println(tcfString2);
						// add pos as well?
					} else if (contentType.startsWith("Word")) {
						String posTagUrl = "service-opennlp/annotate/postag";
						//http://weblicht.sfs.uni-tuebingen.de/rws/service-opennlp/annotate/postag
						tcfString2 = wsClient.callWithTCF(posTagUrl, tcfString);
					}
					// convert to transcription?
					if (tcfString2 != null) {
						TCFtoTranscription tctt = new TCFtoTranscription();
						tctt.setDefaultDuration(1000);
						try {
							TranscriptionImpl nextTrans = tctt.createTranscription(tcfString2);
							//update times based on existing transcription
							if (contentType.equals("Sentence")) {
								TierImpl refTier = (TierImpl) nextTrans.getTierWithId(TCFConstants.SENT);
								if (refTier == null) {
									// message
									return;
								}
								// rename the reference tier
								//refTier.setName(tier.getName());
								// can check number of annotations and / or mappings of orig annotations to sentence id's?
								//
								List origAnns = tier.getAnnotations();
								int numOrigAnns = origAnns.size();
								List procAnns = refTier.getAnnotations();
								int numProcAnns = procAnns.size();
								
								AlignableAnnotation origAnn, procAnn;
								if (numOrigAnns == numProcAnns) {// assume that the original and resulting sentences match
									for (int i = numOrigAnns - 1; i >= 0; i--) {
										origAnn = (AlignableAnnotation) origAnns.get(i);
										procAnn = (AlignableAnnotation) procAnns.get(i);
										// how to effectively update annotation times? Bulldozer mode, direct time slot manipulation?
										if (procAnn.getBeginTimeBoundary() < origAnn.getBeginTimeBoundary()) {
											procAnn.updateTimeInterval(origAnn.getBeginTimeBoundary(), origAnn.getEndTimeBoundary());
										} else {
											// something wrong, wrong estimation when the returned tcf is converted to a transcription
											// TODO
										}
									}
								} else {// different number of sentences
									for (int i = numOrigAnns - 1; i >= 0; i--) {
										if (i < numProcAnns) {
											origAnn = (AlignableAnnotation) origAnns.get(i);
											procAnn = (AlignableAnnotation) procAnns.get(i);
											// how to effectively update annotation times? Bulldozer mode, direct time slot manipulation?
											if (procAnn.getBeginTimeBoundary() < origAnn.getBeginTimeBoundary()) {
												//procAnn.updateTimeInterval(origAnn.getBeginTimeBoundary(), origAnn.getEndTimeBoundary());
											}
										}
									}
								}
								// now merge tiers that are on a lower level then the sentence tier
								List<String> tiersToAdd = new ArrayList<String>();
								List depTiers = refTier.getDependentTiers();
								for(int i = 0; i < depTiers.size(); i++) {
									Tier depTier = (Tier) depTiers.get(i);
									tiersToAdd.add(depTier.getName());
								}
								tiersToAdd.add(0, refTier.getName());
								
								MergeTranscriptionsByAddingCommand command = (MergeTranscriptionsByAddingCommand) 
										ELANCommandFactory.createCommand(trans, ELANCommandFactory.MERGE_TRANSCRIPTIONS_UNDOABLE);
								command.execute(trans, new Object[]{nextTrans, tiersToAdd, Boolean.TRUE, Boolean.TRUE, Boolean.FALSE});
							} else {// token/words
								
							}
						} catch (SAXException sax) {
							
						} catch (IOException ioe) {
							
						}
					}
				}
			} else {
				// message
			}
		}
		
		/**
		 * The input tier is considered to contain sentence level annotations.
		 * 
		 * @param tier the input tier
		 */
		private void processSentenceLevel(TierImpl tier) {
			if (tier == null) {
				ClientLogger.LOG.warning("There is no tier to process.");
			}
			tiersToTCF = new TiersToTCF();
			WebLichtWsClient wsClient = new WebLichtWsClient();
			String tcfString = null;
			// Three possible approaches: 
			// 1. Put all annotations together to one text and upload. Match what is returned with 
			//    the original annotation. This does not work, the sentences can be different from the original 
			//    input, the annotations. 
			// 2. Tokenize the annotations and create tcf ourselves.
			//    
			// 3. Process one annotation at a time in either of the 2 ways
			// This should be in a command

			tcfString = tiersToTCF.toTCFString(tier, "Sentence");
			System.out.println("TCF:");
			System.out.println(tcfString);
			if (tcfString != null) {			
				String tcfString2 = null;
				String posTagUrl = "service-opennlp/annotate/postag";
				//http://weblicht.sfs.uni-tuebingen.de/rws/service-opennlp/annotate/postag
				tcfString2 = wsClient.callWithTCF(posTagUrl, tcfString);
				// convert to transcription?
				if (tcfString2 != null) {
					System.out.println("TCF 2:");
					System.out.println(tcfString2);
					
					TCFtoTranscription tctt = new TCFtoTranscription();
					tctt.setDefaultDuration(1000);
					try {
						TranscriptionImpl nextTrans = tctt.createTranscription(tcfString2);
						//update times based on existing transcription

						TierImpl refTier = (TierImpl) nextTrans.getTierWithId(TCFConstants.SENT);
						if (refTier == null) {
							// message
							return;
						}

						// can check number of annotations and / or mappings of orig annotations to sentence id's?						
						List origAnns = tier.getAnnotations();
						int numOrigAnns = origAnns.size();
						List procAnns = refTier.getAnnotations();
						int numProcAnns = procAnns.size();
						// shift all new annotations beyond the end of the original annotations
						long origEndTime = ((AbstractAnnotation) origAnns.get(numOrigAnns - 1)).getEndTimeBoundary();
						nextTrans.shiftAllAnnotations(origEndTime);
						
						AlignableAnnotation origAnn, procAnn;
						
						for (int i = 0; i < numProcAnns; i++) {
							if (i < numOrigAnns) {
								origAnn = (AlignableAnnotation) origAnns.get(i);
								procAnn = (AlignableAnnotation) procAnns.get(i);

								procAnn.updateTimeInterval(origAnn.getBeginTimeBoundary(), origAnn.getEndTimeBoundary());
							}
						}
						// rename the reference tier
						refTier.setName(tier.getName());
						
						// now merge tiers that are on a lower level then the sentence tier
						List<String> tiersToAdd = new ArrayList<String>();
						List depTiers = refTier.getDependentTiers();
						for(int i = 0; i < depTiers.size(); i++) {
							Tier depTier = (Tier) depTiers.get(i);
							tiersToAdd.add(depTier.getName());
						}
						tiersToAdd.add(0, refTier.getName());
						
						MergeTranscriptionsByAddingCommand command = (MergeTranscriptionsByAddingCommand) 
								ELANCommandFactory.createCommand(trans, ELANCommandFactory.MERGE_TRANSCRIPTIONS_UNDOABLE);
						command.execute(trans, new Object[]{nextTrans, tiersToAdd, Boolean.TRUE, Boolean.TRUE, Boolean.FALSE});
						 
					} catch (SAXException sax) {
						
					} catch (IOException ioe) {
						
					}
				}
			}
		}
	}

}
